/***

The MIT License

Copyright (c) 2018 Teppei Tamra (TAM)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

***/

#include <QMdiSubWindow>
#include "saoriwindow.h"
#include "ui_saoriwindow.h"
#include "saoriapplication.h"
#include "saoriaddaccountdialog.h"
#include "saoriview.h"

SaoriWindow::SaoriWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::SaoriWindow)
{
    ui->setupUi(this);
    connect(ui->actionAdd_Account,&QAction::triggered,this,&SaoriWindow::openAccountDialog);
    updateTimelineList();
}

SaoriWindow::~SaoriWindow()
{
    delete ui;
}

void SaoriWindow::openView(const QStringList viewName)
{
    if (auto view = SaoriView::findView(viewName.at(1),viewName.at(0))) {
        ui->mdiArea->setActiveSubWindow(qobject_cast<QMdiSubWindow*>(view->parent()));
        return;
    }
    QString title = viewName.at(1) + ":" + viewName.at(0);
    auto view = new SaoriView(viewName.at(1),viewName.at(0));
    auto sub = ui->mdiArea->addSubWindow(view);
    sub->setWindowTitle(title);
    sub->show();
}



void SaoriWindow::openAccountDialog()
{
    auto dialog = new SaoriAddAccountDialog(this);
    if (dialog->exec() == QDialog::Accepted) {
        dialog->m_account->setParent(QApplication::instance());
        SaoriApplication::getAccountList()->append(dialog->m_account);
    }
    delete dialog;
}

void SaoriWindow::updateTimelineList()
{
    ui->timelineTree->clear();
    auto *accountItem = new QTreeWidgetItem(ui->timelineTree,QStringList() << tr("Accounts"));
    accountItem->setIcon(0,QIcon(":/icons/ionicons/person-stalker.svg"));
    for (int i = 0;i < SaoriApplication::getAccountList()->count();i ++) {
        auto *item = new QTreeWidgetItem(accountItem,QStringList() << SaoriApplication::getAccountList()->at(i)->name());
        item->setIcon(0,QIcon(":/icons/ionicons/person.svg"));
        accountItem->addChild(item);
        for (auto tl:SaoriApplication::getAccountList()->at(i)->timelineList()) {
            // trick for translation.
            QStringList trtl;
            QString tlText,icon;
            trtl << "home" << "local" << "public" << "notifications";
            switch (trtl.indexOf(tl)) {
            case 0:
                tlText = tr("home");
                icon = ":/icons/ionicons/chatbubbles.svg";
                break;
            case 1:
                tlText = tr("local");
                icon = ":/icons/ionicons/chatbubble.svg";
                break;
            case 2:
                tlText = tr("public");
                icon = ":/icons/ionicons/earth.svg";
                break;
            case 3:
                tlText = tr("notifications");
                icon = ":/icons/ionicons/alert.svg";
                break;
            default:
                tlText = tl;
                break;
            }
            auto tlItem = new QTreeWidgetItem(item,QStringList() << tlText
                                              << SaoriApplication::getAccountList()->at(i)->name()
                                              << tl);
            if (!icon.isEmpty()) tlItem->setIcon(0,QIcon(icon));
            item->addChild(tlItem);
        }
    }
    auto *instanceItem = new QTreeWidgetItem(ui->timelineTree,QStringList() << tr("Instances"));
    for (int i = 0;i < SaoriApplication::getInstanceList()->count();i ++) {
        auto *item = new QTreeWidgetItem(instanceItem,QStringList() << SaoriApplication::getInstanceList()->at(i)->instance().toString());
        instanceItem->addChild(item);
        item->addChild(new QTreeWidgetItem(item,QStringList() << tr("Infomation")));
    }
}

void SaoriWindow::on_timelineTree_itemDoubleClicked(QTreeWidgetItem *item, int)
{
    auto account = SaoriApplication::findAccount(item->text(1));
    if (account) {
        qDebug() << account->instance()->timelineUrl(item->text(2));
        openView(QStringList() << item->text(1) << item->text(2));
    }
}
